@@ -138,9 +138,7 @@ BANNED_FILENAMES = (
 )
 
 
-def cleanup_filename(
-    filename: str, allow_path: bool = False, full_path: bool = False
-) -> str:
+def cleanup_filename(filename: str, allow_path: bool = False, full_path: bool = False) -> str:
     """
     Takes a filename or path and returns a cleaned up version (that is,
     one where invalid characters or tricks like ../ are ignored).
@@ -195,9 +193,7 @@ def cleanup_filename(
     return filename_clean
 
 
-def filename_valid(
-    filename: str, allow_path: bool = False, full_path: bool = False
-) -> bool:
+def filename_valid(filename: str, allow_path: bool = False, full_path: bool = False) -> bool:
     """Returns whether or not a filename is valid."""
     if filename != os.path.normpath(filename):
         return False
@@ -214,12 +210,7 @@ def filename_valid(
             return False
 
     for f in filename.split(os.path.sep):
-        if (
-            f in BANNED_FILENAMES + ("..",)
-            or f.endswith(".")
-            or f != f.strip()
-            or len(f) > 255
-        ):
+        if f in BANNED_FILENAMES + ("..",) or f.endswith(".") or f != f.strip() or len(f) > 255:
             return False
 
     return True
@@ -242,30 +233,57 @@ def natural_compare(a: str, b: str) -> int:
     This returns an integer, satisfying similar constraints to GLib's collate
     functions: 0 if both strings are the same, 1 if a > b, and -1 if a < b.
     """
-    a = a.lower()
-    b = b.lower()
-    if a == b:
+    # Adapted from https://stackoverflow.com/a/8940266
+    convert = lambda text: int(text) if text.isdigit() else text.lower()  # noqa: E731
+    alphanum_key = lambda key: [convert(c) for c in re.split("([0-9]+)", key)]  # noqa: E731
+    sort = sorted([a, b], key=alphanum_key)
+
+    if sort[0] == sort[1]:
         return 0
-
-    # adapted from https://stackoverflow.com/questions/2545532
-    def sort_key(s: str) -> list:
-        out = []
-        for x in s:
-            if x.isdecimal():
-                out.append(int(x))
-            else:
-                try:
-                    out[-1] += x
-                except (IndexError, TypeError):
-                    out.append(x)
-        try:
-            if out[-1] == "":
-                return out[:-1]
-        except (IndexError, TypeError):
-            pass
-        return out
-
-    sort = sorted([a, b], key=sort_key)
-    if sort[0] == a:
+    if sort[0] is a:
         return -1
     return 1
+
+
+def safe_int(value: int | str | None) -> int | None:
+    """Wrapper around int() that handles invalid values gracefully."""
+    if isinstance(value, int):
+        return value
+    elif isinstance(value, str):
+        value = value.strip()
+    elif not value:
+        return 0
+
+    try:
+        return int(value)
+    except ValueError:
+        return 0
+
+
+def safe_float(value: int | float | str | None) -> float | None:
+    """Wrapper around int() that handles invalid values gracefully."""
+    if isinstance(value, float):
+        return value
+    elif isinstance(value, str):
+        value = value.strip()
+    elif not value:
+        return 0.0
+
+    try:
+        return float(value)
+    except ValueError:
+        return 0.0
+
+
+def iter_selection_model(model, position: int = -1, n_items: int = -1):
+    """Iterate over the selected items in a selection model."""
+    if (position >= 0 and n_items < 0) or (n_items >= 0 and position < 0):
+        raise ValueError("Both position and n_items must be set")
+
+    if position >= 0 and n_items >= 0:
+        selection = model.get_selection_range(position, n_items)
+    else:
+        selection = model.get_selection()
+    for i in range(selection.get_size()):
+        pos = selection.get_nth(i)
+        yield model.get_item(pos)
